from typing import Optional

import requests
import typer

from hugie.models import InferenceEndpointConfig
from hugie.settings import Settings
from hugie.utils import TokenNotSetError, format_table, handle_request_error, load_json

settings = Settings()

app = typer.Typer()

headers = {
    "Authorization": f"Bearer {settings.token}",
    "Content-Type": "application/json",
}
API_ERROR_MESSAGE = "An error occured while making the API call"


if not settings.token:
    raise TokenNotSetError


@app.command("ls")
@app.command("list")
def list(
    json: Optional[bool] = typer.Option(
        None, "--json", help="Prints the full output in JSON."
    )
):
    """
    List all the deployed endpoints
    """
    response = requests.get(settings.endpoint_url, headers=headers)

    if json:
        return typer.echo(response.json())

    else:
        data = response.json()

        if data.get("items"):

            names = []
            states = []
            models = []
            revs = []
            url = []

            for item in data["items"]:
                names.append(item["name"])
                states.append(item["status"]["state"])
                models.append(item["model"]["repository"])
                revs.append(item["model"]["revision"])
                url.append(item["status"].get("url"))

                table = format_table(
                    ["Name", "State", "Model", "Revision", "Url"],
                    names,
                    states,
                    models,
                    revs,
                    url,
                )

            typer.secho(table)
        else:
            typer.secho("No endpoints found")


@app.command()
def create(
    data: str = typer.Argument(..., help="Path JSON data to create the endpoint"),
    json: Optional[bool] = typer.Option(
        None, "--json", help="Prints the full output in JSON."
    ),
):
    """
    Create an endpoint

    Args:
        data (str): Path to JSON data to create the endpoint
    """

    data = InferenceEndpointConfig.from_json(data).model_dump()
    name = data["name"]
    vendor = data["provider"]["vendor"]
    repository = data["model"]["repository"]

    try:
        # Try to create the endpoint

        response = requests.post(settings.endpoint_url, headers=headers, json=data)
        response.raise_for_status()

    except (requests.exceptions.HTTPError, requests.exceptions.RequestException) as e:

        handle_request_error(e)

    except Exception as e:

        # Catch all other exceptions

        typer.secho("An unexpected error occured", fg=typer.colors.RED)
        raise SystemExit(e)

    else:

        # If the command succeeds print this output and exit with code 0

        typer.secho(
            f"Endpoint {name} created successfully on {vendor} using {repository}",
            fg=typer.colors.GREEN,
        )

        if json:
            typer.echo(response.json())
        typer.exit(0)


@app.command()
def update(
    name: str = typer.Argument(..., help="Endpoint name"),
    data: str = typer.Argument(..., help="Path to JSON data to update the endpoint"),
    json: Optional[bool] = typer.Option(
        None, "--json", help="Prints the full output in JSON."
    ),
):
    """
    Update an endpoint
    """
    data = InferenceEndpointConfig.from_json(data).model_dump()

    try:
        response = requests.put(
            f"{settings.endpoint_url}/{name}", headers=headers, json=data
        )
        response.raise_for_status()

    except (requests.exceptions.HTTPError, requests.exceptions.RequestException) as e:

        handle_request_error(e)

    except Exception as e:

        # Catch all other exceptions

        typer.secho("An unexpected error occured", fg=typer.colors.RED)
        raise SystemExit(e)

    else:
        typer.secho("Endpoint updated successfully", fg=typer.colors.GREEN)

        if json:
            typer.echo(response.json())

        typer.exit(0)


@app.command()
def delete(
    name: str = typer.Argument(..., help="Endpoint name"),
    force: bool = typer.Option(
        False, "--force", "-f", help="Force deletion without asking user confirmation"
    ),
):
    """
    Delete an endpoint
    """

    if not force:
        delete_endpoint = typer.confirm(
            "Are you sure you want to delete endpoint. Use --force to override"
        )

        if not delete_endpoint:
            typer.echo("Not deleting endpoint")
            raise typer.Abort()

    if force or delete_endpoint:
        try:
            response = requests.delete(
                f"{settings.endpoint_url}/{name}", headers=headers, json={}
            )
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            if response.status_code == 404:
                typer.secho(
                    f"Endpoint {name} does not exist so it cannot be deleted",
                    fg=typer.colors.YELLOW,
                )
                raise SystemExit
            else:
                typer.secho(API_ERROR_MESSAGE, fg=typer.colors.RED)
                raise SystemExit(e)

        typer.secho("Endpoint deleted successfully", fg=typer.colors.GREEN)


@app.command()
def info(
    name: str = typer.Argument(..., help="Endpoint name"),
    json: Optional[bool] = typer.Option(
        None, "--json", help="Prints the full output in JSON."
    ),
):
    """
    Get info about an endpoint
    """

    try:
        response = requests.get(
            f"{settings.endpoint_url}/{name}", headers=headers, json={}
        )
        response.raise_for_status()
    except requests.exceptions.RequestException as e:
        typer.secho(API_ERROR_MESSAGE, fg=typer.colors.RED)
        raise SystemExit(e)

    info = response.json()

    if info.get("name"):

        if json:
            typer.echo(info)

        else:
            names = []
            states = []
            models = []
            revs = []
            url = []

            names.append(info["name"])
            states.append(info["status"]["state"])
            models.append(info["model"]["repository"])
            revs.append(info["model"]["revision"])
            url.append(info["status"].get("url"))

            table = format_table(
                ["Name", "State", "Model", "Revision", "Url"],
                names,
                states,
                models,
                revs,
                url,
            )

            typer.secho(table)
    else:
        typer.secho(f"Endpoint {name} not found")


@app.command()
def logs(name: str = typer.Argument(..., help="Endpoint name")):
    """
    Get logs about an endpoint
    """
    try:
        response = requests.get(
            f"{settings.endpoint_url}/{name}/logs", headers=headers, json={}
        )
        response.raise_for_status()
    except requests.exceptions.RequestException as e:
        typer.secho(API_ERROR_MESSAGE, fg=typer.colors.RED)
        raise SystemExit(e)

    typer.echo(response.content)


@app.command()
def test(
    name: str = typer.Argument(..., help="Endpoint name"),
    inputs: str = typer.Argument(None, help="Input to send the model."),
    input_file: str = typer.Option(
        None, help="Path to JSON file containing queries to send to the model."
    ),
):
    """
    Test an endpoint
    """

    if not inputs and not input_file:
        typer.secho(
            "You must provide either an input string or an input JSON containing your queries",
            fg=typer.colors.RED,
        )
        raise typer.Abort()

    # Get the endpoint url from endpoint info
    try:
        response = requests.get(
            f"{settings.endpoint_url}/{name}", headers=headers, json={}
        )
        response.raise_for_status()
    except requests.exceptions.RequestException as e:
        typer.secho(API_ERROR_MESSAGE, fg=typer.colors.RED)
        raise SystemExit(e)

    info = response.json()
    url = info["status"]["url"]

    if input_file:
        data = load_json(input_file)
    else:
        data = {"inputs": inputs}

    # Send a call to the endpoint
    try:
        response = requests.post(url, headers=headers, json=data)
        response.raise_for_status()
    except requests.exceptions.RequestException as e:
        typer.secho(API_ERROR_MESSAGE, fg=typer.colors.RED)
        raise SystemExit(e)

    typer.echo(response.json())



import os

from pydantic import BaseModel


class Settings(BaseModel):
    """Core settings."""

    base_url: str = "https://api.endpoints.huggingface.cloud"
    endpoint_url: str = f"{base_url}/endpoint"
    provider_url: str = f"{base_url}/provider"
    token: str = os.getenv("HUGGINGFACE_READ_TOKEN")



import json

import typer
import wasabi


def format_table(headers: list, *args):
    """
    Formats a wasabi table ready for printing

    Args:
        column_names (list): List of column names
        *args (list): List of lists containing the data for each column

    """

    data = list(map(tuple, zip(*args)))

    table = wasabi.table(data, header=headers, divider=True)

    return table


def load_json(path: str):
    """
    Loads a JSON file
    """

    try:
        with open(path) as f:
            data = json.load(f)

        return data
    except FileNotFoundError:
        typer.secho(f"File {path} not found", fg=typer.colors.RED)
        raise typer.Exit(code=1)
    except json.decoder.JSONDecodeError:
        typer.secho(f"File {path} is not a valid JSON", fg=typer.colors.RED)
        raise typer.Exit(code=1)



import srsly
import typer

from hugie.models import InferenceEndpointConfig

app = typer.Typer()


@app.command()
def modify(
    path: str,
    accountId: str = typer.Option(
        None, help="ID of the account (for private endpoints)"
    ),
    name: str = typer.Option(None, help="Name of the endpoint"),
    type: str = typer.Option(
        None, help="Type of endpoint, one of ['public', 'protected', 'private']"
    ),
    accelerator: str = typer.Option(
        "None", help="Accelerator to use. One of ['CPU','GPU']"
    ),
    instanceType: str = typer.Option(None),
    instanceSize: str = typer.Option(None),
    minReplica: int = typer.Option(None, help="Minimum number of replicas"),
    maxReplica: int = typer.Option(None, help="Maximum number of replicas"),
    framework: str = typer.Option("huggingface", help="Framework to use"),
    image: str = typer.Option(
        None,
        help="Image to use when deploying model endppint. Must be string representing a valid JSON, e.g. '{'huggingface': {}}'",
    ),
    repository: str = typer.Option(None, help="Name of the hf model repository"),
    revision: str = typer.Option(None, help="Revision of the hf model repository"),
    task: str = typer.Option(None, help="Task of the model"),
    vendor: str = typer.Option(None, help="Vendor to use. One of ['aws','gcp']"),
    region: str = typer.Option(None, help="Vendor specific region, e.g. 'us-east-1'"),
    overwrite: bool = typer.Option(
        False, "--overwrite", "-o", help="Overwrite inpiut file with updated config"
    ),
):
    """
    Modify an existing endpoint config file
    """

    config = InferenceEndpointConfig.from_json(path)

    # Standard configs

    if name:
        config.name = name

    if type:
        config.type = type

    if accountId:
        config.accountId = accountId

    # Model config

    if repository:
        config.model.repository = repository

    if revision:
        config.model.revision = revision

    if framework:
        config.model.framework = framework

    if task:
        config.model.task = task

    if image:
        config.model.image = image

    # Compute config

    if instanceSize:
        config.compute.instanceSize = instanceSize

    if accelerator:
        config.compute.accelerator = accelerator

    if instanceType:
        config.compute.instanceType = instanceType

    # Scaling config

    if minReplica:
        config.scaling.minReplica = minReplica

    if maxReplica:
        config.scaling.maxReplica = maxReplica

    # Provider config

    if vendor:
        config.provider.vendor = vendor

    if region:
        config.provider.region = region

    if overwrite:
        try:
            srsly.write_json(path, config.model_dump())
            typer.secho(f"Updated config at {path}", fg=typer.colors.GREEN)
        except Exception:
            typer.secho(f"Failed to update config at {path}", fg=typer.colors.RED)

    else:
        typer.secho(srsly.json_dumps(config.model_dump()), fg=typer.colors.YELLOW)


if __name__ == "__main__":
    app()



import importlib.metadata
import webbrowser

import typer

from hugie.config import app as config_app
from hugie.endpoint import app as endpoint_app

app = typer.Typer()


@app.command()
def version():
    """Print hugie version"""
    typer.echo(importlib.metadata.version("hugie"))
    typer.Exit(0)


@app.command("ui")
def open_ui():
    """
    Open the Hugging Face Endpoints UI in a browser
    """
    url = "https://ui.endpoints.huggingface.co/"
    typed_url = typer.style(url, fg=typer.colors.BLUE, bold=True)
    typer.echo(f"Opening {typed_url} in your browser...")
    webbrowser.open(url)
    typer.Exit(0)


app.add_typer(endpoint_app, name="endpoint")
app.add_typer(config_app, name="config")

if __name__ == "__main__":
    app()



from pydantic import BaseModel
from pydantic_settings import BaseSettings
from typing import Optional

from hugie.utils import load_json


class ScalingModel(BaseModel):
    minReplica: int = 1
    maxReplica: int = 1


class ComputeModel(BaseModel):

    accelerator: str = None
    instanceSize: str = None
    instanceType: str = None
    scaling: ScalingModel = ScalingModel()


class ModelModel(BaseModel):

    framework: str = None
    image: dict = {"huggingface": {}}
    repository: str = None
    revision: str = None
    task: str = None


class ProviderModel(BaseModel):
    vendor: str = None
    region: str = None


class InferenceEndpointConfig(BaseSettings):
    """
    Config for the inference endpoint
    """

    accountId: Optional[str] = None
    type: str = None
    compute: ComputeModel = ComputeModel()
    model: ModelModel = ModelModel()
    name: str = None
    provider: ProviderModel = ProviderModel()

    @classmethod
    def from_json(self, path: str):
        """
        Load a config from a JSON file.
        """
        config = load_json(path)

        model = ModelModel(
            framework=config["model"]["framework"],
            image=config["model"]["image"],
            repository=config["model"]["repository"],
            revision=config["model"]["revision"],
            task=config["model"]["task"],
        )

        scaling = ScalingModel(
            minReplica=config["compute"]["scaling"]["minReplica"],
            maxReplica=config["compute"]["scaling"]["maxReplica"],
        )

        compute = ComputeModel(
            accelerator=config["compute"]["accelerator"],
            instanceSize=config["compute"]["instanceSize"],
            instanceType=config["compute"]["instanceType"],
            scaling=scaling,
        )

        provider = ProviderModel(
            vendor=config["provider"]["vendor"],
            region=config["provider"]["region"],
        )

        config = InferenceEndpointConfig(
            accountId=config["accountId"],
            type=config["type"],
            compute=compute,
            model=model,
            name=config["name"],
            provider=provider,
        )

        return config
